#ifndef PARABOLICFEM1D_H
#define PARABOLICFEM1D_H

#include <iostream>
#include <fstream>
#include <vector>
#include <set>
#include <string>

#include "sparsematrix.h"
#include "cg.h"
#include "pde1d.h"

/** ParabolicFEM1D:
 *      Provides structures for the assembly and solution 
 *      of a 1D instationary (symmetric) finite element problem
 *      on a given ordered mesh.
 */
class ParabolicFEM1D{

    /* Private Member Variables */

    std::vector<double> meshpoints;                     // The 1D mesh - a vector of grid points
    std::vector<std::vector<double> > solution;         // The solution matrix: a vector at each time step
    std::set<int> dirichletpoints;                      // A set of indizes that indicate the 
                                                        //  dirichlet boundaries:
                                                        //  i in dirichletpoints => u(x_i) = g(x_i)
    
    SparseMatrix A;                                     // System operator matrix A_h
    SparseMatrix M;                                     // Mass matrix M_h
    std::vector<double> F;                              // Right hand side vector b_h
    
    TimedepPDE1D& pde1d;                                // PDE Object specifying the problem to be solved
    
    int timesteps;                                      // Number of time steps in the theta scheme
    double T;                                           // Final time
    double theta;                                       // The theta of the theta-scheme
    
    /* Private Member Functions */
    
    void
    assemble_M();
    
    /** Performs the assembly of A_h and b_h
     */
    void
    assemble_A(int current_timestep);

    /** Fills the solution at time t = 0 with initial condition
     */
    void
    set_initial_condition();
    
public:
    
    /* Public Member Functions */
    
    /** Constructors
     *  (for problems with and without boundary conditions)
     */
    ParabolicFEM1D(std::vector<double> mesh, TimedepPDE1D& pde, 
                   int _timesteps, double _T, double _theta);
    ParabolicFEM1D(std::vector<double> mesh, TimedepPDE1D& pde, 
                   int _timesteps, double _T, double _theta, 
                   std::set<int> dirichlet);
    
    /** Assembles and solves the linear system
     */
    void
    solve();
    
    /** Prints both A_h and b_h to std::cout.
     *  Can be used to debug the system.
     */
    void
    write_system();
    
    /** Write the solution to the specified file
     *  (in format: t x u(t,x))
     *  or prints all to std::cout if no filename is given
     */
    void
    write_solution(std::string filename = std::string());
    
    /** Returns the solution matrix
     *  (necessary if we want to transform the solution 
     *   before printing, as for Black-Scholes)
     */
    std::vector<std::vector<double> >&
    get_solution();

};


//===========================================
// IMPLEMENTATIONS
//===========================================

/********
 * TODO * 
 ********/
ParabolicFEM1D::ParabolicFEM1D(std::vector<double> mesh, TimedepPDE1D& pde, 
                               int _timesteps, double _T, double _theta){}

/********
 * TODO * 
 ********/    
ParabolicFEM1D::ParabolicFEM1D(std::vector<double> mesh, TimedepPDE1D& pde, 
                               int _timesteps, double _T, double _theta,
                               std::set<int> dirichlet){}

/********
 * TODO * 
 ********/
void
ParabolicFEM1D::set_initial_condition()
{
    /* Fill the solution vector at t=0 with initial condition
     */
}
    

/********
 * TODO * 
 ********/    
void
ParabolicFEM1D::assemble_A(int current_timestep)
{
    /* Performs the element-wise assembly of A_h(t) and b_h(t) at the given time step.
     * (Note that we can assume that pde1d.set_time() has been called before,
     * so that the pde1d-object already has the correct time.)
     * Handles the Dirichlet boundary conditions. Depending on where you set u = w + g, 
     * you might need the current_timestep here.
     */
  
}

/********
 * TODO * 
 ********/
void
ParabolicFEM1D::solve()
{
    /* Assembles and solves the system. 
     */
 
}

/********
 * TODO * 
 ********/
void
ParabolicFEM1D::assemble_M()
{
    /* Assembles the mass matrix M_h
     */
}

void
ParabolicFEM1D::write_system()
{
    std::cout << "A = " << std::endl << A << std::endl;
    std::cout << "b = " << std::endl << F << std::endl;
}

void
ParabolicFEM1D::write_solution(std::string filename)
{
    if(filename.size()>0){
        std::ofstream out(filename.c_str());
        if(out.good()){
            double deltaT = T / (double)timesteps;
            for(int t = 0; t <= timesteps; ++t){
                for(size_t i = 0; i < meshpoints.size() ; ++i){
                    out << t*deltaT << " " << meshpoints[i] << " " << solution[t][i] << std::endl;
                }
                out << std::endl;                
            }
        }
        out.close();
        return;
    }
    
    double deltaT = T / (double)timesteps;
    for(int t = 0; t <= timesteps; ++t){
        for(size_t i = 0; i < meshpoints.size() ; ++i){
            std::cout << t*deltaT << " " << meshpoints[i] << " " << solution[t][i] << std::endl;
        }
        std::cout << std::endl;                
    }
}


std::vector<std::vector<double> >&
ParabolicFEM1D::get_solution()
{
    return solution;
}
#endif // PARABOLICFEM1D_H