#include <iostream>
#include <vector>
#include <set>

#include "timedeppoisson1d.h"
#include "parabolicfem1d.h"

using namespace std;

// Global Black-Scholes parameters
// (needed in the functions below)
double r = 0.04;
double sigma = 0.4;
double maturity = 1;
double K = 12;

// Problem right hand side
double
f(double t, double x){
    return 0.;
}

// Problem boundary condition
double
d_bc_fct(double tau, double x){
    double q = 2.*r/(sigma*sigma);
    // x < 0 -> r_1
    if(x < 0){
        return exp(0.5*x*(q-1) + 0.25*(q-1)*(q-1)*tau);
    }
    // x > 0 -> r_2 == 0
    return 0;
}

// Problem initial value function
double
u0_fct(double x){
    double q = 2.*r/(sigma*sigma);
    return x < 0 ? exp(0.5*x*(q-1))*(1-exp(x)) : 0;
}

// Transform heat equation time into Black-Scholes time
double
heat2bs_Time(double tau){
    return maturity - tau/(0.5*sigma*sigma);
}

// Transform heat equation space point into Black-Scholes point
double
heat2bs_Space(double x){
    return K*exp(x);
}

// Transform heat equation solution value into Black-Scholes option price value
double
heat2bs_Value(double y, double tau, double x){
    double q = 2.*r/(sigma*sigma);
    return K*exp(-0.5*x*(q-1) - tau*(0.25*(q-1)*(q-1)+q))*y;
}

int main(){
    
    // (Non-global) Black-Scholes Parameters
//    double S_min = 0.00000000001;
//    double S_max = 20;
    
    // Heat Equation Parameters
//    double x_min = log(S_min/K);
//    double x_max = log(S_max/K);
    double x_min = -5;  // making sure that the strike K is on a grid point
    double x_max = 5;
    double tau_max = 0.5*sigma*sigma*maturity;

    // Initialize 1D mesh with mesh points x_0,...,x_N
    // (=>  S_h = {psi_1,...,psi_(N-1)})
    int N = 200;
    vector<double> mesh(N);    
    double h = (x_max - x_min)/(double)(N-1);
    for(int i = 0; i < N; ++i){
        mesh[i] = x_min + i*h;
    }
    
    // Initialize time discretization parameters
    int timesteps = 50;
    double deltaTau = tau_max/(double)timesteps;
    double theta = 0.5;    
    
    // Initialize Dirichlet boundary conditions
    set<int> d_bc;
    d_bc.insert(0);
    d_bc.insert(N-1);
    
    // Initialize Problem and FEM structure
    TimedepPoisson1D black_scholes(f, d_bc_fct, u0_fct);
    ParabolicFEM1D fem1d(mesh, black_scholes, timesteps, tau_max, theta, d_bc);
    
    // Solve problem
    fem1d.solve();
    
    // Do the back-transformation into Black-Scholes coordinates
    // and write this solution into a file
    vector<vector<double> > y = fem1d.get_solution();
    ofstream outfile("u_bs.txt");
    if(outfile.good()){
        for(size_t tau_ind = 0; tau_ind < y.size(); ++tau_ind){
            for(size_t x_ind = 0; x_ind < y[tau_ind].size(); ++x_ind){
                // Write t S u(t,S) in Black-Scholes-Koordinaten
                double tau = tau_ind*deltaTau;
                double x = x_min + x_ind*h;
                outfile << heat2bs_Time(tau) << " " << heat2bs_Space(x) << " " << heat2bs_Value(y[tau_ind][x_ind], tau, x) << endl;
            }
            outfile << endl;
        }        
    }
    outfile.close();

    return 0;
}