#ifndef ELLIPTICFEM1D_H
#define ELLIPTICFEM1D_H

#include <iostream>
#include <fstream>
#include <vector>
#include <set>
#include <string>

#include "sparsematrix.h"
#include "cg.h"
#include "pde1d.h"

/** EllipticFEM1D:
 *      Provides structures for the assembly and solution 
 *      of a 1D stationary (symmetric) finite element problem
 *      on a given ordered mesh.
 */
class EllipticFEM1D{

    /* Private Member Variables */
    
    std::vector<double> meshpoints;         // The 1D mesh - a vector of grid points
    std::vector<double> solution;           // The solution vector
    std::set<int> dirichletpoints;          // A set of indizes that indicate the 
                                            //  dirichlet boundaries:
                                            //  i in dirichletpoints => u(x_i) = g(x_i)
    
    SparseMatrix A;                         // System matrix A_h
    std::vector<double> F;                  // Right hand side vector b_h
    
    PDE1D& pde1d;                           // PDE Object specifying the problem to be solved
    
    /* Private Member Functions */
    
     /** Performs the assembly of A_h and b_h
      */
    void
    assemble();                            
    
public:
    
    /* Public Member Functions */
    
    /** Constructors
     *  (for problems with and without boundary conditions)
     */
    EllipticFEM1D(std::vector<double> mesh, PDE1D& pde);
    
    EllipticFEM1D(std::vector<double> mesh, PDE1D& pde, std::set<int> dirichlet);
    
    /** Assembles and solves the linear system
     */
    void
    solve();
    
    /** Prints both A_h and b_h to std::cout.
     *  Can be used to debug the system.
     */
    void
    write_system();
    
    /** Write the solution to the specified file
     *  (in format: x u(x))
     *  or print the vector to std::cout if no filename is given
     */
    void
    write_solution(std::string filename = std::string());

};


//===========================================
// IMPLEMENTATIONS
//===========================================

/********
 * TODO * 
 ********/
EllipticFEM1D::EllipticFEM1D(std::vector<double> mesh, PDE1D& pde){}

/********
 * TODO * 
 ********/   
EllipticFEM1D::EllipticFEM1D(std::vector<double> mesh, PDE1D& pde, std::set<int> dirichlet){}
    
    
/********
 * TODO * 
 ********/
void
EllipticFEM1D::assemble()
{
    /* Performs the element-wise assembly.
     * Handles the Dirichlet boundary conditions.
     */
}

/********
 * TODO * 
 ********/
void
EllipticFEM1D::solve()
{    
    /* Assembles and solves the system. 
     */
}

void
EllipticFEM1D::write_system()
{
    std::cout << "A = " << std::endl << A << std::endl;
    std::cout << "b = " << std::endl << F << std::endl;
}

void
EllipticFEM1D::write_solution(std::string filename)
{
    if(filename.size()>0){
        std::ofstream out(filename.c_str());
        if(out.good()){
            for(size_t i = 0; i < meshpoints.size() ; ++i){
                out << meshpoints[i] << " " << solution[i] << std::endl;
            }
        }
        out.close();
        return;
    }
    std::cout << "u = " << std::endl << solution << std::endl;
}


#endif // ELLIPTICFEM1D_H