#ifndef OPERATORS_H
#define OPERATORS_H 1

#include "densematrix.h"
#include "sparsematrix.h"
#include "vector.h"
#include <vector>

/* This header contains Matrix-Vector and Vector-Vector operations
 * for the matrix types
 *  - DenseMatrix
 *  - SparseMatrix
 * and the vector types
 *  - Vector 
 *  - std::vector<double>
 */
 
 
//==============================
// Operators for Vector
//==============================

Vector
operator*(const DenseMatrix &m, const Vector &v){
    
    Vector ret(m.numRows());
    for(int i = 0; i< m.numRows(); ++i){
        
        ret[i] = 0.0;
        for(int j = 0; j < m.numCols(); ++j){
            ret[i] += m(i,j)*v[j];
        }
    }
    return ret;
}

Vector
operator-(const Vector &v1, const Vector &v2){
    Vector ret(v1.size());
    for(int i = 0; i < v1.size(); ++i){
        ret[i] = v1[i] - v2[i];
    }
    return ret;
}

Vector
operator+(const Vector &v1, const Vector &v2){
    Vector ret(v1.size());
    for(int i = 0; i < v1.size(); ++i){
        ret[i] = v1[i] + v2[i];
    }
    return ret;
}



Vector
operator*(const double scalar, const Vector &v){
    //std::cout << "v.size() = " << v.size() << std::endl;
    Vector ret(v.size());
    for(int i = 0; i < v.size(); ++i){
        ret[i] = v[i] * scalar;
    }
    return ret;
}

double
operator*(const Vector &v1, const Vector &v2){
    double ret=0.0;
    for(int i = 0; i < v1.size(); ++i){
        ret+=v1[i]*v2[i];
    }

    return ret;
}


Vector
operator*(SparseMatrix &m, const Vector &v){
    Vector ret(m.numRows());
    for(std::map<std::pair<int,int>,double>::iterator iterator = m.data.begin(); iterator != m.data.end(); iterator++) {
        ret[(iterator->first).first] += m((iterator->first).first,(iterator->first).second)*v[(iterator->first).second];
    }
    return ret;
}

//=====================================
// Operators for std::vector<double>
//=====================================

std::vector<double>
operator*(const DenseMatrix &m, const std::vector<double> &v){
    
    std::vector<double> ret(m.numRows());
    for(int i = 0; i< m.numRows(); ++i){
        
        ret[i] = 0.0;
        for(int j = 0; j < m.numCols(); ++j){
            ret[i] += m(i,j)*v[j];
        }
    }
    return ret;
}

std::vector<double>
operator-(const std::vector<double> &v1, const std::vector<double> &v2){
    std::vector<double> ret(v1.size());
    for(size_t i = 0; i < v1.size(); ++i){
        ret[i] = v1[i] - v2[i];
    }
    return ret;
}

std::vector<double>
operator+(const std::vector<double> &v1, const std::vector<double> &v2){
    std::vector<double> ret(v1.size());
    for(size_t i = 0; i < v1.size(); ++i){
        ret[i] = v1[i] + v2[i];
    }
    return ret;
}



std::vector<double>
operator*(const double scalar, const std::vector<double> &v){
    //std::cout << "v.size() = " << v.size() << std::endl;
    std::vector<double> ret(v.size());
    for(size_t i = 0; i < v.size(); ++i){
        ret[i] = v[i] * scalar;
    }
    return ret;
}

double
operator*(const std::vector<double> &v1, const std::vector<double> &v2){
    double ret=0.0;
    for(size_t i = 0; i < v1.size(); ++i){
        ret+=v1[i]*v2[i];
    }

    return ret;
}


std::vector<double>
operator*(SparseMatrix &m, const std::vector<double> &v){
    std::vector<double> ret(m.numRows());
    for(std::map<std::pair<int,int>,double>::iterator iterator = m.data.begin(); iterator != m.data.end(); iterator++) {
        ret[(iterator->first).first] += m((iterator->first).first,(iterator->first).second)*v[(iterator->first).second];
    }
    return ret;
}

std::ostream& 
operator<< (std::ostream& out, const std::vector<double> & v){
    for(size_t i = 0; i < v.size(); ++i){
        out << v[i] << std::endl;
    }
    
    return out;
}

#endif //OPERATORS_H