#ifndef POISSON1D_H
#define POISSON1D_H

#include "pde1d.h"

// Define type for 1D function (-> shorthand notation)
typedef double (*Fct1D)(double);

/** Poisson1D:
 *      Problem class that implements a 1D Laplace Operator
 *      
 *      Optionally with 
 *       - (non-homogeneous) Dirichlet boundary conditions
 *       - (non-homogeneous) initial values for non-stationary problems
 */
class Poisson1D : public PDE1D{
        
    Fct1D f;            // Right hand side function
    Fct1D d_bc;         // Dirichlet boundary condition function
    Fct1D u0_fct;       // Initial value condition function
    

public:
    
    /* Public Member Functions */
    
    /** Constructors
     */
    Poisson1D(Fct1D _f);
    Poisson1D(Fct1D _f, Fct1D _d_bc);
    Poisson1D(Fct1D _f, Fct1D _d_bc, Fct1D _u0_fct);
    
    /** Assembles A^(k) and b^(k) on each interval [x_i, x_(i+1)]
     */
    void
    assemble_on_element(double a, double b, double A[2][2], double b[2]);
    
    /** Returns the Dirichlet boundary condition at point x
     *  (overloads the PDE1D version)
     */
    double
    dirichlet_bc(double x);

    /** Returns the initial condition at point x.
     *  (overloads the PDE1D version)
     */
    double
    initial_condition(double x);

};


//===========================================
// IMPLEMENTATIONS
//===========================================

Poisson1D::Poisson1D(Fct1D _f) : f(_f){}
Poisson1D::Poisson1D(Fct1D _f, Fct1D _d_bc) : f(_f), d_bc(_d_bc){}
Poisson1D::Poisson1D(Fct1D _f, Fct1D _d_bc, Fct1D _u0_fct) : f(_f), d_bc(_d_bc), u0_fct(_u0_fct){}

/********
 * TODO * 
 ********/
void
Poisson1D::assemble_on_element(double x_left, double x_right, double A[2][2], double b[2])
{
    /* Fill A and b with the correct values
     */
}

double
Poisson1D::dirichlet_bc(double x){
    // If d_bc exists, return function value
    if(d_bc != NULL){
        return d_bc(x);
    }
    // Else return default value 0
    return 0;
};

double
Poisson1D::initial_condition(double x){
    if(u0_fct != NULL){
        return u0_fct(x);
    }
    return 0;
};

#endif // POISSON_H